#include <fggl/debug/debug.h>

#include <imgui.h>
#include <imgui_impl_glfw.h>
#include <imgui_impl_opengl3.h>

using fggl::gfx::Window;
using fggl::debug::DebugUI;

DebugUI::DebugUI(std::shared_ptr<fggl::gfx::GlfwWindow>& win) : m_visible(false) {
	IMGUI_CHECKVERSION();
	ImGui::CreateContext();

	ImGuiIO& io = ImGui::GetIO();
	io.IniFilename = nullptr;

	ImGui_ImplGlfw_InitForOpenGL(win->handle(), true);
	ImGui_ImplOpenGL3_Init("#version 130");
}

DebugUI::~DebugUI() {
	ImGui_ImplOpenGL3_Shutdown();
	ImGui_ImplGlfw_Shutdown();
	ImGui::DestroyContext();
}

void DebugUI::frameStart() {
	// frame step
	ImGui_ImplOpenGL3_NewFrame();
	ImGui_ImplGlfw_NewFrame();
	ImGui::NewFrame();
}

void DebugUI::draw() {
	for ( auto& [name, data] : m_windows) {
		if ( data.m_visible ) {
			data.m_callback( &data.m_visible );
		}
	}

	ImGui::Render();
	if ( m_visible ) {
		ImGui_ImplOpenGL3_RenderDrawData(ImGui::GetDrawData());
	}
}
