/*
 * ${license.title}
 * Copyright (C) 2022 ${license.owner}
 * ${license.mailto}
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "fggl/math/triangulation.hpp"

namespace fggl::math {

	/**
	 * Fast Triangulation for convex polygons.
	 */
	void fan_triangulation(const PolygonVertex& polygon, data::Mesh2D &mesh) {
		assert(polygon.size() >= 3);

		// add the first two points to the mesh
		auto firstIdx = mesh.add_vertex(polygon[0]);
		auto prevIdx = mesh.add_vertex(polygon[1]);

		// deal with the indices
		const auto nTris = polygon.size() - 2;
		for (auto i = 0; i < nTris; i++) {
			mesh.add_index(firstIdx);
			mesh.add_index(prevIdx);

			auto currIdx = mesh.add_vertex(polygon[i + 2]);
			mesh.add_index(currIdx);
			prevIdx = currIdx;
		}
	}

} // namespace fggl::math

