//
// Created by webpigeon on 20/11/2021.
//

#ifndef FGGL_HEIGHTMAP_H
#define FGGL_HEIGHTMAP_H

#include <cstdint>

namespace fggl::data {

	constexpr std::size_t heightMaxX = 255;
	constexpr std::size_t heightMaxZ = 255;
	constexpr float heightSeaLevel = 0.0f;

	struct HeightMap {
		constexpr static const char name[] = "Heightmap";
		float heightValues[heightMaxX * heightMaxZ];

		void clear() {
			for (float &heightValue : heightValues) {
				heightValue = heightSeaLevel;
			}
		}

		[[nodiscard]]
		inline float getValue(std::size_t x, std::size_t z) const {
			return heightValues[x * heightMaxZ + z];
		}

		inline void setValue(std::size_t x, std::size_t z, float value) {
			heightValues[x * heightMaxZ + z] = value;
		}
	};

	inline int idx(int x, int z, int zMax) {
		return x * zMax + z;
	}

	void generateHeightMesh(const data::HeightMap *heights, data::Mesh &mesh);
}

#endif //FGGL_HEIGHTMAP_H
