//
// Created by webpigeon on 23/10/2021.
//

#ifndef FGGL_ECS3_MODULE_H
#define FGGL_ECS3_MODULE_H

#include <string>
#include <map>
#include <memory>

#include <fggl/ecs3/types.hpp>

namespace fggl::ecs3 {

	class Module {
		public:
			virtual ~Module() = default;

			[[nodiscard]] virtual std::string name() const = 0;

			virtual void onLoad(ModuleManager &manager, TypeRegistry &tr) {};

			virtual void onUpdate();
			virtual void onFrameStart();
			virtual void onFrameEnd();
	};

	class ModuleManager {
		public:
			explicit ModuleManager(TypeRegistry &types) : m_types(types), m_modules() {}

			~ModuleManager() = default;

			template<typename C, typename... Args>
			std::shared_ptr<C> load(Args &... args) {
				auto ptr = std::make_shared<C>(args...);
				m_modules[ptr->name()] = ptr;

				ptr->onLoad(*this, m_types);
				//spdlog::info("loaded ECS module: {}", ptr->name());
				return ptr;
			}

			template<typename C>
			void onAdd(const callback_t &cb) {
				m_types.callbackAdd(Component<C>::typeID(), cb);
			}

			void onUpdate() {
				for (auto &[id, ptr] : m_modules) {
					ptr->onUpdate();
				}
			}

			void onFrameStart() {
				for (auto &[id, ptr] : m_modules) {
					ptr->onFrameStart();
				}
			}

			void onFrameEnd() {
				for (auto &[id, ptr] : m_modules) {
					ptr->onFrameEnd();
				}
			}

		private:
			TypeRegistry &m_types;
			std::map<std::string, std::shared_ptr<Module>> m_modules;
	};

}

#endif //FGGL_ECS3_MODULE_H
