#ifndef FGGL_GFX_COMPAT_HPP
#define FGGL_GFX_COMPAT_HPP
/**
 * Window management Calls.
 *
 * This shouldn't be exposed to the demo app, but the ECS we're using isn't smart enouph to allow us to
 * abstract this yet. It's next thing on the list, but this branch is about cleaning up OpenGL not about
 * extending our ECS.
 *
 * Should be removed when the engine has suitable abstractions in place.
 */

#include <memory>

#include <utility>
#include <fggl/gfx/window.hpp>
#include <fggl/ecs3/ecs.hpp>

namespace fggl::gfx {

	//
	// fake module support - allows us to still RAII
	//
	struct ecsGlfwModule : ecs3::Module {

		inline explicit
		ecsGlfwModule(std::shared_ptr<fggl::input::Input> inputs) : context( std::make_shared<GlfwContext>(std::move(inputs))) {
		}

		inline
		std::unique_ptr<GlfwWindow> createWindow(const std::string &title) {
			std::unique_ptr<GlfwWindow> window = std::make_unique<GlfwWindow>(context);
			window->title(title);
			return std::move(window);
		}

		void onUpdate() override {
			context->pollEvents();
		}

		[[nodiscard]]
		std::string name() const override {
			return "gfx::glfw";
		}

		private:
			std::shared_ptr<GlfwContext> context;

	};

}

#endif
