#ifndef FGGL_GFX_RENDERER_H
#define FGGL_GFX_RENDERER_H

#include <fggl/data/model.hpp>
#include <fggl/ecs3/ecs.hpp>
#include <fggl/gfx/ogl/backend.hpp>
#include <fggl/gfx/ogl/shader.hpp>

#include "fggl/gfx/ogl4/models.hpp"
#include "fggl/gfx/ogl4/canvas.hpp"

#include <fggl/gfx/paint.hpp>
#include <fggl/gfx/renderer.hpp>

#include <memory>
#include <vector>

namespace fggl::gfx {

	enum GlRenderType {
		triangles = GL_TRIANGLES,
		triangle_strip = GL_TRIANGLE_STRIP
	};

	struct GlRenderToken {
		constexpr static const char name[] = "RenderToken";
		GLuint vao;
		GLuint buffs[2];
		GLuint idxOffset;
		GLsizei idxSize;
		GLuint pipeline;
		GLuint restartVertex;
		GlRenderType renderType = triangles;
	};

	class GlMeshRenderer {
		public:
			using token_t = GlRenderToken;

			token_t upload(fggl::data::Mesh &mesh);

			void render(ecs3::World &ecs, ecs3::entity_t camera, float dt);

			float total;
	};

	class OpenGL4Backend : public Graphics {
		public:
			explicit OpenGL4Backend(const Window &owner);
			~OpenGL4Backend() override = default;

			// copy bad
			OpenGL4Backend(const OpenGL4Backend&) = delete;
			OpenGL4Backend& operator=(const OpenGL4Backend&) = delete;

			void clear() override;
			void resize(int width, int height) override;

			void draw2D(const Paint &paint) override;
			void drawScene(ecs3::World& world) override;

		private:
			std::unique_ptr<ogl4::StaticModelRenderer> m_modelRenderer;
			std::unique_ptr<ogl4::CanvasRenderer> m_canvasRenderer;
			std::unique_ptr<ShaderCache> m_cache;
	};

	using OpenGL4 = OpenGL4Backend;

// specialisation hooks
	using MeshRenderer = GlMeshRenderer;
	using MeshToken = GlRenderToken;

};  // namespace fggl::gfx

#endif
