#ifndef FGGL_GFX_GLFW_INPUT_H
#define FGGL_GFX_GLFW_INPUT_H

#include <memory>
#include <string>

#include <fggl/gfx/window.hpp>
#include <fggl/input/input.hpp>

namespace fggl::gfx {

	class GlfwInputManager {
		public:
			static GlfwInputManager &instance() {
				static GlfwInputManager *instance = new GlfwInputManager();
				return *instance;
			}

			inline void setup(std::shared_ptr<input::Input> input) {
				m_inputs = input;
			}

			inline void frame() {
				m_inputs->frame(0.0f);
			}

			inline bool alive() {
				return m_inputs != nullptr;
			}

			inline input::MouseInput &mouse() {
				return m_inputs->mouse;
			}

			inline input::KeyboardInput &keyboard() {
				return m_inputs->keyboard;
			}

			inline input::GamepadInput &gamepads() {
				return m_inputs->gamepads;
			}

			inline void onMouseMove(float x, float y) {
				if (m_inputs != nullptr) {
					m_inputs->mouse.axis(input::MouseAxis::X, x);
					m_inputs->mouse.axis(input::MouseAxis::Y, y);
				}
			}

			inline void onMouseScroll(float x, float y) {
				if (m_inputs != nullptr) {
					m_inputs->mouse.axis(input::MouseAxis::SCROLL_X, x);
					m_inputs->mouse.axis(input::MouseAxis::SCROLL_Y, y);
				}
			}

			inline void onMouseButton(int btn, bool state) {
				if (m_inputs != nullptr) {
					m_inputs->mouse.button((fggl::input::MouseButton) btn, state);
				}
			}

			inline void onKeyEvent(int scancode, bool state) {
				if (m_inputs != nullptr) {
					m_inputs->keyboard.set(scancode, state);
				}
			}

		private:
			std::shared_ptr<input::Input> m_inputs;

	};

};

#endif
