//
// Created by webpigeon on 20/11/2021.
//

#ifndef FGGL_CAMERA_INPUT_H
#define FGGL_CAMERA_INPUT_H

#include <fggl/ecs3/ecs.hpp>
#include <fggl/math/types.hpp>

namespace fggl::input {

	constexpr float ROT_SPEED = 0.05f;
	constexpr float PAN_SPEED = 0.05f;
	constexpr math::mat4 MAT_IDENTITY(1.0f);

	struct FreeCamKeys {
		constexpr const static char name[] = "FreeCameraKeys";
		scancode_t forward;
		scancode_t backward;
		scancode_t left;
		scancode_t right;
		scancode_t rotate_cw;
		scancode_t rotate_ccw;
	};

	/**
	 * Process the camera based on rotation around a fixed point.
	 *
	 * @param ecs the world that contains the camera
	 * @param input the input module to read the mouse location from
	 * @param cam the ID of the camera entity
	 */
	void process_arcball(fggl::ecs3::World &ecs, const Input &input, fggl::ecs::entity_t cam);

	/**
	 * Process free (floating) camera movement.
	 *
	 * @param ecs the world that contains the camera
	 * @param input the input module to read the mouse location from
	 * @param cam the ID of the camera entity
	 */
	void process_freecam(fggl::ecs3::World &ecs, const Input &input, fggl::ecs::entity_t cam);

	/**
	 * Input processing for moving the camera when the mouse is close to the edge of the screen.
	 *
	 * This function deals with ensuring the camera moves if the user moves their mouse button close
	 * to the edge of the screen. It will apply this as movement to the camera entity passed in as
	 * an argument.
	 *
	 * @param ecs the world that contains the camera
	 * @param input the input module to read the mouse location from
	 * @param cam the ID of the camera entity
	 */
	void process_edgescroll(fggl::ecs3::World &ecs, const Input &input, fggl::ecs::entity_t cam);

}

#endif //FGGL_CAMERA_INPUT_H
