//
// Created by webpigeon on 20/11/2021.
//

#ifndef FGGL_SCENE_H
#define FGGL_SCENE_H

#include <memory>
#include <string>
#include <unordered_map>

namespace fggl::scenes {

	class Scene {
		public:
			virtual ~Scene() = default;

			virtual void setup() = 0;
			virtual void cleanup() = 0;

			virtual void update() = 0;
			virtual void render() = 0;
	};

	class SceneManager {
		public:
			SceneManager() = default;

			void create(const std::string &name, std::shared_ptr<Scene> scene);
			void activate(const std::string &name);

			inline void update() {
				if (m_active == nullptr) {
					return;
				}
				m_active->update();
			}

			inline void render() {
				if (m_active == nullptr) {
					return;
				}
				m_active->render();
			}

		private:
			std::shared_ptr<Scene> m_active;
			std::unordered_map<std::string, std::shared_ptr<Scene>> m_scenes;
	};

}

#endif //FGGL_SCENE_H
